/**********************************************************************
 * Copyright (c) 2008, 2012 Intel Corporation and others.
 * All rights reserved. This content is made available under
 * the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html       
 * $Id: ThreadEventsContext.java,v 1.3 2010/05/11 16:30:04 jwest Exp $
 *
 * Contributors:
 * Intel - Initial contribution
 **********************************************************************/ 

package org.eclipse.hyades.loaders.trace;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.eclipse.hyades.models.trace.TRCObject;
import org.eclipse.hyades.models.trace.TRCThread;
import org.eclipse.hyades.models.trace.TRCThreadDeadAndNotifyJoinedEvent;
import org.eclipse.hyades.models.trace.TRCThreadInterruptThreadEvent;
import org.eclipse.hyades.models.trace.TRCThreadNotifyAllEvent;
import org.eclipse.hyades.models.trace.TRCThreadNotifyEvent;
import org.eclipse.hyades.models.trace.TRCThreadStartThreadEvent;

public class ThreadEventsContext {

	private HashMap<TRCObject, TRCThreadStartThreadEvent> threadObj2CallStartEventMap = new HashMap<TRCObject, TRCThreadStartThreadEvent>();
	
	/** Each thread has a unique ID and Java object associated with it. Map allows to
	 * define thread ID by its ObjRefID. */
	private HashMap<TRCObject, TRCThread> object2ThreadMap = new HashMap<TRCObject, TRCThread>();
	
	/** Map defines which thread currently owns monitor of object with certain objectIDRef */
	private HashMap<TRCObject, TRCThread> monitor2OwnerMap = new HashMap<TRCObject, TRCThread>();
	
	private HashMap<TRCThread, TRCThreadDeadAndNotifyJoinedEvent> thread2ThreadDeadNotifyEventMap = new HashMap<TRCThread, TRCThreadDeadAndNotifyJoinedEvent>();
	
	private HashMap<TRCObject, List<TRCThreadNotifyEvent>> monitor2NotifyMap = new HashMap<TRCObject, List<TRCThreadNotifyEvent>>();
	private HashMap<TRCObject, TRCThreadNotifyAllEvent> monitor2NotifyAllMap = new HashMap<TRCObject, TRCThreadNotifyAllEvent>();
	
	
	private HashMap<TRCThread, TRCThreadInterruptThreadEvent> thread2InterruptionMap = new HashMap<TRCThread, TRCThreadInterruptThreadEvent>();

	private Map<Long, List<Object>> threadSleepData = Collections.synchronizedMap(new HashMap<Long, List<Object>>());
	
	private ThreadDeadlockDetector threadDeadlockDetector = new ThreadDeadlockDetector(); 
	
	public Map<TRCObject, TRCThreadStartThreadEvent> getThreadObj2CallStartEventMap() {
		return threadObj2CallStartEventMap;
	}

	public Map<TRCObject, TRCThread> getObject2ThreadMap() {
		return object2ThreadMap;
	}

	public Map<TRCObject, TRCThread> getMonitor2OwnerMap() {
		return monitor2OwnerMap;
	}
	
	public Map<TRCThread, TRCThreadDeadAndNotifyJoinedEvent> getThread2ThreadDeadNotifyEventMap() {
		return thread2ThreadDeadNotifyEventMap;
	}

	public Map<TRCObject, List<TRCThreadNotifyEvent>> getMonitor2NotifyMap() {
		return monitor2NotifyMap;
	}

	public Map<TRCObject, TRCThreadNotifyAllEvent> getMonitor2NotifyAllMap() {
		return monitor2NotifyAllMap;
	}

	public Map<TRCThread, TRCThreadInterruptThreadEvent> getThread2InterruptionMap() {
		return thread2InterruptionMap;
	}

	/** Given a threadID, return a list of entries that contain the time at which the thread began sleeping (only applicable for threads that are sleeping) */
	public List/*<Object> */<Object> getThreadSleepData(long threadId) {
		if(threadSleepData == null) {
			threadSleepData = Collections.synchronizedMap(new HashMap<Long, List<Object>>()); 
		}
		
		List<Object> threadEntries = threadSleepData.get(new Long(threadId));
		if(threadEntries == null) {
			threadEntries = Collections.synchronizedList(new ArrayList/*<Object>*/<Object>());
			threadSleepData.put(new Long(threadId), threadEntries);
		}	
		
		return threadEntries;
	}

	/** Returns the class responsible for maintaining event data and detecting deadlocks */
	public ThreadDeadlockDetector getThreadDeadlockDetector() {
		return threadDeadlockDetector;
	}
}
