/**********************************************************************
 * Copyright (c) 2003, 2011 IBM Corporation and others.
 * All rights reserved.   This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * $Id: XMLmethodExitLoader.java,v 1.18 2007-03-08 21:24:33 jkubasta Exp $
 *
 * Contributors:
 * IBM - Initial API and implementation
 * 
 * $Id: XMLmethodExitLoader.java,v 1.18 2007-03-08 21:24:33 jkubasta Exp $
 **********************************************************************/
package org.eclipse.hyades.loaders.trace;

import org.eclipse.hyades.loaders.internal.binary.v1.BinaryTraceConstants;
import org.eclipse.hyades.loaders.trace.TraceUtils.InvocationInfo;
import org.eclipse.hyades.loaders.util.HierarchyContext;
import org.eclipse.hyades.loaders.util.LoadersUtils;
import org.eclipse.hyades.loaders.util.LookupServiceExtensions;
import org.eclipse.hyades.models.hierarchy.TRCAgent;
import org.eclipse.hyades.models.trace.TRCFullMethodInvocation;
import org.eclipse.hyades.models.trace.TRCFullTraceObject;
import org.eclipse.hyades.models.trace.TracePackage;
import org.eclipse.hyades.models.util.ModelDebugger;


/**
 * @author slavescu
 *
 */
public class XMLmethodExitLoader extends TraceMethodBaseLoader {
    //~ Static fields/initializers -----------------------------------------------------------------

    //  protected static final String TICKET = "ticket";
    //	protected static final String STACK_DEPTH = "stackDepth";
    protected static final String OVERHEAD = "overhead";
    protected static final String CPU_TIME = "cpuTime";
    protected static final String CONSUMED_CPU_TIME = "consumedCpuTime";

    //~ Instance fields ----------------------------------------------------------------------------

    
    
    protected double overhead;
    protected double cpuTime;
	public void addAttribute(String name, String value) {
        if (!loadToModel) {
            return;
        }

        switch (LoadersUtils.getHashCode(name)) {
        case TraceConstants.OVERHEAD_int:
//            overhead = Double.parseDouble(value);
            overhead = LoadersUtils.parseTraceTime(value);

            break;
        case TraceConstants.CPU_TIME_int:
        case TraceConstants.CONSUMED_CPU_TIME_int:
//            cpuTime = Double.parseDouble(value);
        	cpuTime = LoadersUtils.parseTraceTime(value);

            break;

        default:
            super.addAttribute(name, value);

            break;
        }
    }

	/** Thread CPU attribute is for us, otherwise pass to parent*/
	public void addAttribute(int con, String s) {

		switch(con) {
			
			case BinaryTraceConstants.THREAD_CPU_TIME:
			cpuTime = LoadersUtils.parseTraceTime(s);
			break;
			
			default:
			super.addAttribute(con, s);
		}
	}

    public void addYourselfInContext() {
        if (!loadToModel) {
            return;
        }
		if(context.isFilterOn())
		{
			if(context.getFilterEngine().isFiltered(TracePackage.eINSTANCE.getTRCMethod_Name(),LoadersUtils.getLookUpKey(methodIdRef)))
			{
				return;
			}
		}

        _cs = (CallStackPerThread) LookupServiceExtensions.getInstance().locate(context, CallStackPerThread.class, threadIdRef);

        if ((_cs == null) || _cs.isEmpty()) {
            return;
        }

        _invocationPool = _cs.invocationPool;

        theProcess = getProcess();
        
        dispatchProcessMode(ProcessSteps.ALL);
    }

    public void initialize(HierarchyContext context, String name) {
        loadToModel = context.isLoadToModel();

        if (!loadToModel) {
            return;
        }

        super.initialize(context, name);
        overhead = 0;
        cpuTime=0;
    }
    
    protected void processEF(int step) {
        super.processEF(step);

        _fullInvocation = ((InvocationInfo) _cs.peek()).getMethodInvocation();

        _fullInvocation.setMethodExitTraceOrder(TraceOrderManager.getInstance().getNextTraceOrderId(theProcess));
        
        if (_fullInvocation != null) {
            theThread = _fullInvocation.getThread();
            _fullInvocation.setOrigExitTime(time);
            _fullInvocation.setExitTime(createDeltaTime());
            _fullInvocation.setOverhead(overhead);
            addInputOutputValues();

            synchronized (_REMOTE_INVOCATION_RESOLUTION_LOCK) {
    			updateForwardInvokes(_fullInvocation);
    		}

			if(_useUpdateStatisticalInfoOld)
				updateStatisticalInfo();
			else
				updateStatisticalInfoNew();
            _invocationPool.release((InvocationInfo) _cs.pop());
        } else {
            // ignore method exit if there is no method entry
        }
    }

    protected void processES(int step) {
        super.processES(step);

        //		theThread = getThreadByIdRef(theProcess);
        //		theMethod = fullInvocation.getMethod();
        //		theObject = fullInvocation.getOwningObject();
        //		theClass = theMethod.getDefiningClass();
        updateStatisticalInfoOnly();
    }

    protected void updateStatisticalInfoNew() {
        try {
			double childrenCumulativTime = 0;
			for (int i = _fullInvocation.getInvokes().size(); i-->0;) {
				TRCFullMethodInvocation element = (TRCFullMethodInvocation) _fullInvocation.getInvokes().get(i);
				if(isAsynchronous(_fullInvocation, element)) { // bugzilla 119991
					// Parent method and child method are in different threads.
					// Assume asynchronous call; disregard from this calculation
					continue;
				}
				if(element.getExitTime()>0)
					childrenCumulativTime = childrenCumulativTime+element.getExitTime()-element.getEntryTime()-element.getOverhead();
				
			}
			
			_deltaBaseTime = _fullInvocation.getExitTime() - _fullInvocation.getEntryTime() - _fullInvocation.getOverhead() - childrenCumulativTime;
			
			_invokerMethod = _fullInvocation.getMethod();
			_invokerObject = (TRCFullTraceObject) _fullInvocation.getOwningObject();
			_invokerClass = _fullInvocation.getMethod().getDefiningClass();

			_invokerObjectClass = getExtendedClass(_invokerObject,_invokerClass);
			
			updateCPUTime();

			//update base and cumulative using deltaBaseTime, invokerObject and invokerClass
			updateTimeStatistics();
			
			if(_fullInvocation.getInvokedBy()!=null && _fullInvocation.eContainer() != _fullInvocation.getInvokedBy().eContainer())
				updateBaseTimeIfRequired(_fullInvocation,_fullInvocation.getInvokedBy());
        } catch (Exception e) {
			ModelDebugger.log(e);
        }
    }
    
    /**
     * Return true if the parent and child invocations were executed asynchronously;
     * false otherwise.
     * 
     * @author IBM
     */
    private boolean isAsynchronous(TRCFullMethodInvocation parentInvocation, TRCFullMethodInvocation childInvocation) {
		/*
		 * Scenarios for an synchronous and asynchronous transaction.
		 * 1) Synchronous - Two events on the same host and same thread.
		 * 2) Asynchronous - Two events on the same host and different threads and one of the 
		 * events has an invocation context in which the agent ID is same as on the other event's TRCAgent.
		 * 3) Asynchronous - Two events on different hosts.
		 */
//    	String parentInvocationHost = parentInvocation.getThread().getProcess().getAgent().getAgentProxy().getProcessProxy().getNode().getName();
//		String childInvocationHost = childInvocation.getThread().getProcess().getAgent().getAgentProxy().getProcessProxy().getNode().getName();
//		if (parentInvocationHost.equalsIgnoreCase(childInvocationHost)) {
			/* Same Hostnames */
			if (parentInvocation.getThread().getId() == childInvocation.getThread().getId()) {
				/* Same Thread */
				return false;
			} else {
				/* Different Threads - now verify if the agents are the same */
				TRCAgent parentAgent = parentInvocation.getThread().getProcess().getAgent();
				TRCAgent childAgent = childInvocation.getInvokedBy().getThread().getProcess().getAgent();
				if (parentAgent.getRuntimeId() == childAgent.getRuntimeId())
					return true;
				else
					return false;
			}
//		} else {
//			/* Different Hostnames */
//			return true;
//		}
	}

	protected void updateStatisticalInfo() {
        try {
            int size = _fullInvocation.getInvokes().size();

            if (size > 0) {
                //this "invocation" has children
                _lastChildExitTime = ((TRCFullMethodInvocation) _fullInvocation.getInvokes().get(size - 1)).getExitTime();
                _deltaBaseTime = _fullInvocation.getExitTime() - _lastChildExitTime - _fullInvocation.getOverhead();
            } else {
                //this "invocation" doesn't have children
                _deltaBaseTime = _fullInvocation.getExitTime() - _fullInvocation.getEntryTime() - _fullInvocation.getOverhead();
            }

			_invokerMethod = _fullInvocation.getMethod();
			_invokerObject = (TRCFullTraceObject) _fullInvocation.getOwningObject();
			_invokerClass = _fullInvocation.getMethod().getDefiningClass();

			_invokerObjectClass = getExtendedClass(_invokerObject,_invokerClass);
			
			updateCPUTime();
            //update base and cumulative using deltaBaseTime, invokerObject and invokerClass
            updateTimeStatistics();

            //			p.setCurrentTime(createTimeDouble());
        } catch (Exception e) {
			ModelDebugger.log(e);
        }
    }

    /**
	 * 
	 */
	protected void updateCPUTime() {
		if(cpuTime!=0)
		{
			try {
				double correctedCPUTime = cpuTime;//-overhead;
				if(_fullInvocation!=null)
					_fullInvocation.setCpuTime(correctedCPUTime);
				_invokerMethod.setTotalCpuTime(_invokerMethod.getTotalCpuTime()+correctedCPUTime);
				_invokerClass.setTotalCpuTime(_invokerClass.getTotalCpuTime()+correctedCPUTime);
				_invokerClass.getPackage().setTotalCpuTime(_invokerClass.getPackage().getTotalCpuTime()+correctedCPUTime);
				getProcess().setTotalCpuTime(getProcess().getTotalCpuTime()+correctedCPUTime);
				
			} catch (Exception e) {
				ModelDebugger.log(e);
			}
		}
	}

	protected void updateStatisticalInfoOnly() {
        try {
            InvocationInfo currentInvocation = (InvocationInfo) _cs.peek();
            double exitTime = createDeltaTime();

            _lastChildExitTime = currentInvocation.getLastChildExitTime();
            double lastChildOverhead = currentInvocation.getLastChildOverhead(); // bugzilla 123628

            if (_lastChildExitTime > 0) {
                _deltaBaseTime = exitTime - _lastChildExitTime - overhead + lastChildOverhead; 
            } else {
                _deltaBaseTime = exitTime - currentInvocation.getEntryTime() - overhead;
            }

            _invokerObject = currentInvocation.getObject();

            _invokerClass = currentInvocation.getTheClass();

            _invokerObjectClass = currentInvocation.getObjectClass();

            _invokerMethod = currentInvocation.getMethod();
            
            updateCPUTime();
            //update base and cumulative time using deltaBaseTime, invokerObject and invokerClass
            updateTimeStatistics();

            //remove current invocation
            _invocationPool.release((InvocationInfo) _cs.pop());

            if (!_cs.isEmpty()) {
                //update current invocation parent 
                InvocationInfo invoker = (InvocationInfo) _cs.peek();

                if (invoker != null) {
                    invoker.setLastChildExitTime(exitTime);
                    invoker.setLastChildOverhead(overhead); // bugzilla 123628
                }
            }

            theProcess.setLastEventTime(exitTime);
        } catch (Exception e) {
			ModelDebugger.log(e);
        }
    }
}
