/**********************************************************************
 * Copyright (c) 2003, 2012 IBM Corporation and others.
 * All rights reserved.   This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * $Id: HierarchyContext.java,v 1.2 2008-01-24 02:28:16 apnan Exp $
 *
 * Contributors:
 * IBM - Initial API and implementation
 **********************************************************************/
package org.eclipse.hyades.loaders.util;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.eclipse.hyades.models.hierarchy.TRCAgent;
import org.eclipse.hyades.models.hierarchy.TRCMonitor;
import org.eclipse.hyades.models.hierarchy.TRCNode;
import org.eclipse.hyades.models.hierarchy.TRCProcessProxy;
import org.eclipse.hyades.models.hierarchy.extensions.SimpleSearchQuery;
import org.eclipse.hyades.models.hierarchy.util.internal.ImportFilterEngine;


public class HierarchyContext extends AgentProxyContext{
    //~ Instance fields ----------------------------------------------------------------------------

    protected List<Object> globalForwardReferences;
	protected String contextURI;
    protected TRCAgent agent;
    protected TRCMonitor monitor;
    protected TRCNode node;
    protected TRCProcessProxy processProxy;
    protected Map<Object, Object> customData;
	protected SimpleSearchQuery importFilter;
	protected ImportFilterEngine filterEngine;
	/**
	 *
	 */

	public HierarchyContext() {
		super();
	}

    //~ Methods ------------------------------------------------------------------------------------

    /**
     * Sets the agent.
     * @param agent The agent to set
     */
    public void setAgent(TRCAgent agent) {
        this.agent = agent;
    }

    /**
     * @return TRCAgent
     */
    public TRCAgent getAgent() {
        return agent;
    }

    /**
     * Sets the globalForwardReferences.
     * @param globalForwardReferences The globalForwardReferences to set
     */
    public void setGlobalForwardReferences(List<Object> globalForwardReferences) {
        this.globalForwardReferences = globalForwardReferences;
    }

    /**
     * @return List
     */
    public List<Object> getGlobalForwardReferences() {
    	if(globalForwardReferences==null)
			globalForwardReferences = new ArrayList<Object>();
        return globalForwardReferences;
    }

    /**
     * Sets the monitor.
     * @param monitor The monitor to set
     */
    public void setMonitor(TRCMonitor monitor) {
        this.monitor = monitor;
    }

    /**
     * @return TRCMonitor
     */
    public TRCMonitor getMonitor() {
        return monitor;
    }

    /**
     * Sets the node.
     * @param node The node to set
     */
    public void setNode(TRCNode node) {
        this.node = node;
    }

    /**
     * @return TRCNode
     */
    public TRCNode getNode() {
        return node;
    }

    /**
     * Sets the processProxy.
     * @param processProxy The processProxy to set
     */
    public void setProcessProxy(TRCProcessProxy processProxy) {
        this.processProxy = processProxy;
    }

    /**
     * @return TRCProcessProxy
     */
    public TRCProcessProxy getProcessProxy() {
        return processProxy;
    }
	/**
	 * @return
	 */
	public String getContextURI() {
		return contextURI;
	}

	/**
	 * @param string
	 */
	public void setContextURI(String string) {
		contextURI = string;
	}

	/**
	 * Map for storing custom data, each loader can add it's own key/value pairs.
	 * @return
	 */
	public Map<Object, Object> getCustomData()
	{
		if(customData==null)
			customData = new HashMap<Object, Object>();
		return customData;
	}

	/**
	 * @param map
	 */
	public void setCustomData(Map<Object, Object> map)
	{
		customData = map;
	}

	/**
	 * @return
	 */
	public boolean isFilterOn() {
		return importFilter!=null;
	}

	/**
	 * @return
	 */
	public ImportFilterEngine getFilterEngine() {
		if(filterEngine==null)
		{
			filterEngine = new ImportFilterEngine(importFilter,null);
		}
		return filterEngine;
	}
	
	/**
	 * @param importFilter The importFilter to set.
	 */
	public void setImportFilter(SimpleSearchQuery importFilter) {
		this.importFilter = importFilter;
	}

	public void cleanUp() {
		super.cleanUp();
		this.globalForwardReferences = null;
		this.agent = null;
		this.monitor = null;
		this.node = null;
		this.processProxy = null;
		if(this.customData!=null)
			this.customData.clear();
		this.customData = null;
		this.importFilter = null;
		this.filterEngine = null;
	}

}
