/*******************************************************************************
 * Copyright (c) 2011, 2012 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * $Id: 
 * 
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/

package org.eclipse.hyades.trace.ui.internal.export;

import java.io.File;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.eclipse.hyades.models.hierarchy.TRCAgent;
import org.eclipse.hyades.models.hierarchy.TRCAgentProxy;
import org.eclipse.hyades.models.hierarchy.TRCMonitor;
import org.eclipse.hyades.models.hierarchy.TRCNode;
import org.eclipse.hyades.models.hierarchy.TRCProcessProxy;
import org.eclipse.hyades.models.hierarchy.TRCTIProfilerCollectionMode;
import org.eclipse.hyades.trace.internal.ui.PDContentProvider;
import org.eclipse.hyades.trace.ui.HyadesUtil;
import org.eclipse.hyades.trace.ui.internal.wizard.TraceWizardMessages;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.TreePath;
import org.eclipse.jface.viewers.TreeSelection;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.accessibility.AccessibleAdapter;
import org.eclipse.swt.accessibility.AccessibleEvent;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.TableItem;
import org.eclipse.swt.widgets.Text;
import org.eclipse.tptp.platform.common.ui.internal.CommonUIImages;
import org.eclipse.tptp.platform.common.ui.trace.internal.CommonUITraceImages;

public class TraceExportWizardPage1 extends WizardPage {

	private Combo _destinationNameField;
	private String _prevNameFieldValue;
	
	private Button _destinationBrowseButton;
	private IStructuredSelection _selection;
	
	private List<TableItem> _agentList;

	protected TraceExportWizardPage1(IStructuredSelection selection) {
		super(TraceExportWizardPage1.class.getName());
		
		_selection = selection;
		
		setTitle(TraceWizardMessages.EXPORT_WIZARD_WINDOW_HEADER);
		setDescription(TraceWizardMessages.EXPORT_WIZARD_WINDOW_DESC);
		
		setImageDescriptor(CommonUIImages.INSTANCE.getImageDescriptor(CommonUITraceImages.IMG_UI_WZ_EXPORT_FILE));
		
	}

	/** Scan through the list of monitors provided by PDContentProvider, and locate the TRCAgents */
	@SuppressWarnings("rawtypes")
	private List<TRCAgent> getAgentList() {
		List<TRCAgent> result = new ArrayList<TRCAgent>();
		
		List l = PDContentProvider.getMonitors();
		if(l == null) return result;
		// Monitors
		for(Iterator it = l.iterator(); it.hasNext();) {
			Object o = it.next();
			if(o instanceof TRCMonitor) {
				TRCMonitor m = (TRCMonitor)o;
				
				// Nodes				
				if(m.getNodes() == null) return result;
				for(TRCNode n : m.getNodes()) {
					
					// Process Proxies
					if(n.getProcessProxies() == null) return result;
					for(TRCProcessProxy pp : n.getProcessProxies()) {
						
						// Agent Proxies
						if(pp.getAgentProxies() == null) return result;
						for(TRCAgentProxy ap : pp.getAgentProxies()) {
							TRCAgent aa = ap.getAgent();
							result.add(aa);
						}
					}
				}
			}
		}
		
		return result;
	}
	
	/** Has the user right-clicked an existing agent from the view? If so, return that agent. */
	private TRCAgent getSelectedAgentFromUI() {
		ISelection is = _selection;
		
		if(_selection == null) return null;
		
		TreeSelection ts = (TreeSelection)is;
		TreePath[] tpa = ts.getPaths();
		if(tpa == null) return null;
		
		for(int x = 0 ; x < tpa.length; x++) {
			TreePath tp = tpa[x];

			for(int y = 0; y < tp.getSegmentCount(); y++ ) {
				Object o = tp.getSegment(y);
				
				if(o instanceof TRCAgentProxy) {
					TRCAgentProxy ap = (TRCAgentProxy)o;
					TRCAgent aa = ap.getAgent();

					// If the user selected a thread agent, we don't currently handle thread, so ignore the selection
					boolean skipAgent = false;
					if(ap != null) {
						TRCTIProfilerCollectionMode m = ap.getTiCollectionMode();
						if(m == TRCTIProfilerCollectionMode.THREAD_CONTENTION || m == TRCTIProfilerCollectionMode.THREAD_NO_CONTENTION) {
							skipAgent = true;
						}
					}
					
					if(!skipAgent) {
						return aa;
					} else {
						continue;
					}
				}
			}			
		}
		
		return null;
	}
	
	private void createAgentSelectionComposite(Composite parent) {
		
		Composite c = new Composite(parent, SWT.NONE);
		GridData cData = new GridData();
		cData.horizontalAlignment = GridData.FILL;
		cData.grabExcessHorizontalSpace = true;
		c.setLayoutData(cData);
	
		GridLayout layout = new GridLayout();
		layout.numColumns = 1;
		c.setLayout(layout);
		
		TRCAgent selectedAgent = getSelectedAgentFromUI();
		
		List<TRCAgent> list = null;
		if(selectedAgent != null) {
			list = new ArrayList<TRCAgent>();
			list.add(selectedAgent);

		} else {
			list  = getAgentList();
		}
		
		Table t = new Table(c, SWT.BORDER | SWT.CHECK);
		GridData tData = new GridData();
		tData.horizontalAlignment = GridData.FILL;
		tData.grabExcessHorizontalSpace = true;
		t.setLayoutData(tData);
		t.setToolTipText(TraceWizardMessages.EXPORT_WIZARD_AGENT_LIST_DESCRIPTOR);
		
		TableColumn tc1 = new TableColumn(t, SWT.CENTER);
		tc1.setText(TraceWizardMessages.EXPORT_WIZARD_PROF_AGENT);
		
		t.setHeaderVisible(true);
		
		final List<TableItem> tableItems = new ArrayList<TableItem>();
		for(TRCAgent l : list) {

			// Skip thread profiling agents, as we do not currently handle them
			if(l.getAgentProxy() != null) {
				TRCTIProfilerCollectionMode m = l.getAgentProxy().getTiCollectionMode();
				if(m == TRCTIProfilerCollectionMode.THREAD_CONTENTION || m == TRCTIProfilerCollectionMode.THREAD_NO_CONTENTION) {
					continue;
				}				
			}
			
			String label = HyadesUtil.getAgentName(l.getAgentProxy()) + " -- "+TraceWizardMessages.EXPORT_WIZARD_PID+": "+l.getAgentProxy().getProcessProxy().getPid(); //$NON-NLS-1$ //$NON-NLS-2$
			TableItem item = new TableItem(t, SWT.NONE);
			item.setText(new String[] { label });
			item.setData(l);
			
			TRCTIProfilerCollectionMode mode = l.getAgentProxy().getTiCollectionMode();
			if(mode == null || mode == TRCTIProfilerCollectionMode.UNKNOWN)  {
				mode = TraceOutputUtil.calcTICollectionMode(l.getAgentProxy(), l);
			}
			
			if(mode == TRCTIProfilerCollectionMode.HEAP_ALLOC_SITES || mode == TRCTIProfilerCollectionMode.HEAP_NO_ALLOC_SITES) {
				item.setImage(CommonUITraceImages.INSTANCE.getImage(CommonUITraceImages.IMG_UI_HEAP_TYPE));
			} else {
				item.setImage(CommonUITraceImages.INSTANCE.getImage(CommonUITraceImages.IMG_UI_EXEC_TYPE));
			}
			tableItems.add(item);
			
			if(selectedAgent != null && selectedAgent == l) {
				item.setChecked(true);
			}
			
		}
		
		if(tableItems.size() > 0) {
			tc1.pack();
		} else {
			tc1.setWidth(500);
		}
		
		_agentList = tableItems;
		
		t.addSelectionListener(new SelectionListener() {

			public void widgetDefaultSelected(SelectionEvent e) {					
			}

			public void widgetSelected(SelectionEvent e) {
				for(TableItem i : tableItems) {
					i.setChecked(false);
				}
				if(e.item instanceof TableItem) {
					TableItem t = (TableItem)e.item;
					t.setChecked(true);
				}
				
			}
			
		});
		
		EWListener l = new EWListener();
		t.addListener(SWT.Selection, l);
			
			
	}
	
	public void createControl(Composite parent) {
		Composite group = new Composite(parent, SWT.NONE);
		GridLayout layout = new GridLayout();
		layout.numColumns = 1;
		group.setLayout(layout);
		
		Text text = new Text(group, SWT.READ_ONLY);
		text.setBackground(group.getBackground());
		text.setText(TraceWizardMessages.EXPORT_WIZARD_PROF_DATA_TO_EXPORT);
		
		createAgentSelectionComposite(group);
		
		new Label(group, SWT.NONE); // vertical spacer
		
		Label exportLabel = new Label(group, SWT.NONE);
		exportLabel.setText(TraceWizardMessages.EXPORT_WIZARD_EXPORT_DEST);
		
		
		Composite exportGroup = new Composite(group, SWT.NONE);
		
			GridData exportGroupOuter = new GridData();
			exportGroupOuter.horizontalAlignment = GridData.FILL;
			exportGroupOuter.grabExcessHorizontalSpace = true;
		exportGroup.setLayoutData(exportGroupOuter);
		
		// Inner composite, export group:
		
		GridLayout expLayout = new GridLayout();
		expLayout.numColumns = 3;
		exportGroup.setLayout(expLayout);
		Label preText = new Label(exportGroup, SWT.NONE);
		
		if(TraceExportWizard.isBinaryOutputFormat()) {
			preText.setText(TraceWizardMessages.EXPORT_WIZARD_TRACE_FILE_BIN);
		} else {
			preText.setText(TraceWizardMessages.EXPORT_WIZARD_TRACE_FILE_XML);
		}
		
		_destinationNameField = new Combo(exportGroup, SWT.SINGLE | SWT.BORDER);
		GridData destNameFieldData = new GridData();
		destNameFieldData.horizontalAlignment = GridData.FILL;
		destNameFieldData.grabExcessHorizontalSpace = true;		
		_destinationNameField.setLayoutData(destNameFieldData);
		EWListener l = new EWListener();
		_destinationNameField.addListener(SWT.Selection, l);
		_destinationNameField.addListener(SWT.Modify, l);
		_destinationNameField.setToolTipText(TraceWizardMessages.EXPORT_WIZARD_FILE_FIELD_DESCRIPTOR);
		_destinationNameField.getAccessible().addAccessibleListener(new AccessibleAdapter() {
			public void getName(AccessibleEvent e) {
				e.result = TraceWizardMessages.EXPORT_WIZARD_FILE_FIELD_DESCRIPTOR;
			}
		});
		
		_destinationBrowseButton = new Button(exportGroup, SWT.PUSH);
		_destinationBrowseButton.setText(TraceWizardMessages.EXPORT_WIZARD_BROWSE);
		_destinationBrowseButton.addSelectionListener(new BrowseButtonSelectionListener());
		
		_destinationBrowseButton.setToolTipText(TraceWizardMessages.EXPORT_WIZARD_FILE_FIELD_DESCRIPTOR);;
		
		setControl(group);
		
	}

    
    class BrowseButtonSelectionListener implements SelectionListener {

		public void widgetDefaultSelected(SelectionEvent arg0) {
			
		}

		public void widgetSelected(SelectionEvent arg0) {
	        String currentSource = _destinationNameField.getText();
			
	        FileDialog dlg = new FileDialog(_destinationNameField.getShell());
	        
	        String extension = ""; //$NON-NLS-1$
	        if(TraceExportWizard.isBinaryOutputFormat()) {
	        	extension = "trcbin"; //$NON-NLS-1$
	        } else {
	        	extension = "trcxml"; //$NON-NLS-1$
	        }
	        
	        dlg.setFilterPath(currentSource);
	        
	        dlg.setFilterExtensions(new String[] { "*."+extension }); //$NON-NLS-1$ 
	        dlg.open();
			
	        String fileName = dlg.getFileName();
	
	        if ((fileName != null) && !fileName.equals("")) { //$NON-NLS-1$
	        	if(!fileName.contains(".")) { //$NON-NLS-1$
	        		fileName += "."+extension; //$NON-NLS-1$
	        	}
	        	
	        	_destinationNameField.setText(dlg.getFilterPath() + File.separator + fileName);
	        }
			
		}
    	
    }
    
    @Override
    public boolean isPageComplete() {
    	boolean isChecked = false;
    	
    	// Something must be checked
    	if(_agentList != null) {
    		for(TableItem i : _agentList) {
    			if(i.getChecked())  {
    				isChecked= true;
    				break;
    			}
    		}
    	}
    	
    	if(!isChecked) {
    		return false;
    	}
    	
    	// Destination name field must contain valid value
    	if(_destinationNameField != null) {
    		String text = _destinationNameField.getText(); 
    		if(text != null && text.trim().length() > 0) {
    			return true;
    		}
    	}
    	
    	return false;
    }
    
    @Override
    public boolean canFlipToNextPage() {
    	return false;
    	
    }

    /** Listen on the appropriate fields and call updateButtons() when needed. */
    class EWListener implements Listener {

		public void handleEvent(Event e) {
			boolean update = false;
			
			if(e.widget == _destinationNameField) {
				if(_prevNameFieldValue == null || _prevNameFieldValue.length() == 0) {
					update = true;
				}
				
				if(_destinationNameField.getText() == null || _destinationNameField.getText().trim().length() == 0) {
					update = true;					
				}
				_prevNameFieldValue = _destinationNameField.getText();
				
				
			} else {
				update = true;
			}
			
			if(update) {
				getWizard().getContainer().updateButtons();
			}
			
		}
    	
    }
    
    public String getFilename() {
    	return _destinationNameField.getText();
    }
    
    public TRCAgent getSelectedAgent() {
    	for(TableItem i : _agentList) {
    		if(i.getChecked()) {
    			return (TRCAgent)i.getData();
    		}
    	}
    	return null;
    }
    
    
}
