/*******************************************************************************
 * Copyright (c) 2005, 2012 Intel Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Hoang M Nguyen, Intel - Initial API and Implementation
 * $Id: TPTPOSCalls.h,v 1.13 2010-01-29 21:05:41 jwest Exp $
 *******************************************************************************/


/**
 *********************************************************
 *
 * @file      tptpOSCalls
 *
 * @brief
 *    This provides a generic interface to OS-specific system calls
 *
 *
 *********************************************************/

#ifndef __TPTPOSCALLS_H
#define __TPTPOSCALLS_H


#include "tptp/TPTPSupportTypes.h"


extern const char OS_SPECIFIC_NAME_SEPARATOR ;


#ifdef __cplusplus
extern "C" {
#endif


char * getGlobalUniqueId() ;


int compareIgnoreCase(const char *pStr1, const char *pStr2) ;


void * loadTheModule(const char *pLibName) ;


/** An optional function for AIX; allows the developer to set the stackSize of the thread. */
#ifdef _AIX
	int  startNewThreadAIXStackSize(RUN_FUNC_TYPE  pFunc, void * pParmBlock, TID *pThreadId, HANDLE *pTheadHandle, int stackSize) ;
#endif

int  startNewThread(RUN_FUNC_TYPE  pFunc, void * pParmBlock, TID *pThreadId, HANDLE *pTheadHandle) ;
int  tptpStartThread(RUN_FUNC_TYPE  pFunc, void * pParmBlock, TID *pThreadId, HANDLE *pTheadHandle) ;


PID  getCurrentlyRunningProcessId() ;

#ifdef MVS
  PID  getCurrentlyRunningThreadId();
#else
  TID  getCurrentlyRunningThreadId() ;
#endif

void printCurrentSysError() ;


void  tptpSleep(int milliseconds) ;

#define UNKNOWN_EXE "unknown"

char *getExecutableName();

#ifdef _WIN32

// Wrappers around stdio. The wrapped versions correctly handle long path 
// names. It'd be nice to only wrap fopen, the resulting FILE* should be
// useable with the regular functions, but there are issues in sharing
// FILE* objects across .dll boundaries:
//
//  ref: http://msdn.microsoft.com/en-us/library/ms235460%28v=vs.80%29.aspx
//
// This is an issue because most of the AC's .dlls statically link the C 
// runtime. Dynamic linking requires a manifest be generated for each project.

int tptp_fclose( FILE *fp );
int tptp_feof( FILE *fp );
int tptp_ferror( FILE *fp );
int tptp_fflush( FILE *fp );
char *tptp_fgets( char *buf, int max, FILE* fp );
FILE *tptp_fopen( const char *filename, const char *mode );
int tptp_fprintf( FILE *fp, const char *fmt, ... );
size_t tptp_fread( void *dst, size_t el, size_t count, FILE *fp );
void tptp_frewind( FILE *fp );
int tptp_fseek( FILE *fp, long ofs, int origin );
long tptp_ftell( FILE *fp );
size_t tptp_fwrite( const void *src, size_t el, size_t count, FILE *fp );

#else

#define tptp_fclose fclose
#define tptp_feof feof
#define tptp_ferror ferror
#define tptp_fflush fflush
#define tptp_fgets fgets
#define tptp_fopen fopen
#define tptp_fprintf fprintf
#define tptp_fread fread
#define tptp_frewind rewind
#define tptp_fseek fseek
#define tptp_ftell ftell
#define tptp_fwrite fwrite

#endif

#ifdef __cplusplus
}
#endif


#endif /* __TPTPOSCALLS_H*/


