/*****************************************************************************
 * Copyright (c) 1997, 2012, Intel and IBM Corporation.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Intel Corporation - Initial API and implementation
 *
 * $Id$ 
 *****************************************************************************/
//

#include <stdlib.h>
#include <string.h>

#include "MIEInstruction.h"
#include "MethodInstrumentor.h"
#include "ValidityChecks.h"
#include "MIEInstructionFactory.h"

#define IS_PSEUDO_MNEM(mnem) (mnem >= MNM_FIRST_PSEUDO && mnem < MNM_LAST_PSEUDO)

using namespace Martini::MIE;

//////////////////////////////////////////////////////////////////////////
// class CMIEInstruction implementation

CMIEInstruction::CMIEInstruction(CMethodInstrumentor* pInstrumentor)
: m_pInstrumentor(pInstrumentor), m_pfgNode(NULL), m_vpOperands(NULL), m_uiOperandsCount(0),
    m_uiTargetOffset(0), m_uiOriginalOffset(0), m_uiTargetsCount(0), 
    m_vuiTargetOffsets(NULL), m_pReadOnlyInst(NULL), m_uiActualOffset(0),
    m_uiLabel(0)
{
}

CMIEInstruction::~CMIEInstruction()
{
    unsigned int uiOpCount = GetOperandsCount();
    if (uiOpCount > 0)
    {
        // free operands
        unsigned int i;
        for (i = 0; i < uiOpCount; ++i)
        {
            DeallocateOperand(m_vpOperands[i]);
            delete m_vpOperands[i];
        }
        delete [] m_vpOperands;
    }
    // free offsets array of switch instructions
    if (m_uiTargetsCount > 0)
    {
        delete [] m_vuiTargetOffsets;
    }
    // free read-only wrapper
    if (m_pReadOnlyInst != NULL)
    {
        delete m_pReadOnlyInst;
    }
}

const EMnemonic CMIEInstruction::GetMnemonic() const
{
    return m_mnem;
}

TResult CMIEInstruction::GetMnemonicName(char * out) {
	switch(m_mnem) {
		case MNM_NOP:
			strcpy(out, "MNM_NOP");
			break;
		case MNM_ACONST_NULL:
			strcpy(out, "ACONST_NULL");
			break;
		case MNM_ICONST_M1:
			strcpy(out, "MNM_ICONST_M1");
			break;
		case MNM_ICONST_0:
			strcpy(out, "MNM_ICONST_0");
			break;
		case MNM_ICONST_1:
			strcpy(out, "MNM_ICONST_1");
		case MNM_ICONST_2:
			strcpy(out, "MNM_ICONST_2");
			break;
		case MNM_ICONST_3:
			strcpy(out, "MNM_ICONST_3");
			break;
		case MNM_ICONST_4:
			strcpy(out, "MNM_ICONST_4");
			break;
		case MNM_ICONST_5:
			strcpy(out, "MNM_ICONST_5");
			break;
		case MNM_LCONST_0:
			strcpy(out, "MNM_LCONST_0");
			break;
		case MNM_LCONST_1:
			strcpy(out, "MNM_LCONST_1");
			break;
		case MNM_FCONST_0:
			strcpy(out, "MNM_FCONST_0");
			break;
		case MNM_FCONST_1:
			strcpy(out, "MNM_FCONST_1");
			break;
		case MNM_FCONST_2:
			strcpy(out, "MNM_FCONST_2");
			break;
		case MNM_DCONST_0:
			strcpy(out, "MNM_DCONST_0");
			break;
		case MNM_DCONST_1:
			strcpy(out, "MNM_DCONST_1");
			break;
		case MNM_BIPUSH:
			strcpy(out, "MNM_BIPUSH");
			break;
		case MNM_SIPUSH:
			strcpy(out, "MNM_SIPUSH");
			break;
		case MNM_LDC:
			strcpy(out, "MNM_LDC");
			break;
		case MNM_LDC_W:
			strcpy(out, "MNM_LDC_W");
			break;
		case MNM_LDC2_W:
			strcpy(out, "MNM_LDC2_W");
			break;
		case MNM_ILOAD:
			strcpy(out, "MNM_ILOAD");
			break;
		case MNM_LLOAD:
			strcpy(out, "MNM_LLOAD");
			break;
		case MNM_FLOAD:
			strcpy(out, "MNM_FLOAD");
			break;
		case MNM_DLOAD:
			strcpy(out, "MNM_DLOAD");
			break;
		case MNM_ALOAD:
			strcpy(out, "MNM_ALOAD");
			break;
		case MNM_ILOAD_0:
			strcpy(out, "MNM_ILOAD_0");
			break;
		case MNM_ILOAD_1:
			strcpy(out, "MNM_ILOAD_1");
			break;
		case MNM_ILOAD_2:
			strcpy(out, "MNM_ILOAD_2");
			break;
		case MNM_ILOAD_3:
			strcpy(out, "MNM_ILOAD_3");
			break;
		case MNM_LLOAD_0:
			strcpy(out, "MNM_LLOAD_0");
			break;
		case MNM_LLOAD_1:
			strcpy(out, "MNM_LLOAD_1");
			break;
		case MNM_LLOAD_2:
			strcpy(out, "MNM_LLOAD_2");
			break;
		case MNM_LLOAD_3:
			strcpy(out, "MNM_LLOAD_3");
			break;
		case MNM_FLOAD_0:
			strcpy(out, "MNM_FLOAD_0");
			break;
		case MNM_FLOAD_1:
			strcpy(out, "MNM_FLOAD_1");
			break;
		case MNM_FLOAD_2:
			strcpy(out, "MNM_FLOAD_2");
			break;
		case MNM_FLOAD_3:
			strcpy(out, "MNM_FLOAD_3");
			break;
		case MNM_DLOAD_0:
			strcpy(out, "MNM_DLOAD_0");
			break;
		case MNM_DLOAD_1:
			strcpy(out, "MNM_DLOAD_1");
			break;
		case MNM_DLOAD_2:
			strcpy(out, "MNM_DLOAD_2");
			break;
		case MNM_DLOAD_3:
			strcpy(out, "MNM_DLOAD_3");
			break;
		case MNM_ALOAD_0:
			strcpy(out, "MNM_ALOAD_0");
			break;
		case MNM_ALOAD_1:
			strcpy(out, "MNM_ALOAD_1");
			break;
		case MNM_ALOAD_2:
			strcpy(out, "MNM_ALOAD_2");
			break;
		case MNM_ALOAD_3:
			strcpy(out, "MNM_ALOAD_3");
			break;
		case MNM_IALOAD:
			strcpy(out, "MNM_IALOAD");
			break;
		case MNM_LALOAD:
			strcpy(out, "MNM_LALOAD");
			break;
		case MNM_FALOAD:
			strcpy(out, "MNM_FALOAD");
			break;
		case MNM_DALOAD:
			strcpy(out, "MNM_DALOAD");
			break;
		case MNM_AALOAD:
			strcpy(out, "MNM_AALOAD");
			break;
		case MNM_BALOAD:
			strcpy(out, "MNM_BALOAD");
			break;
		case MNM_CALOAD:
			strcpy(out, "MNM_CALOAD");
			break;
		case MNM_SALOAD:
			strcpy(out, "MNM_SALOAD");
			break;
		case MNM_ISTORE:
			strcpy(out, "MNM_ISTORE");
			break;
		case MNM_LSTORE:
			strcpy(out, "MNM_LSTORE");
			break;
		case MNM_FSTORE:
			strcpy(out, "MNM_FSTORE");
			break;
		case MNM_DSTORE:
			strcpy(out, "MNM_DSTORE");
			break;
		case MNM_ASTORE:
			strcpy(out, "MNM_ASTORE");
			break;
		case MNM_ISTORE_0:
			strcpy(out, "MNM_ISTORE_0");
			break;
		case MNM_ISTORE_1:
			strcpy(out, "MNM_ISTORE_1");
			break;
		case MNM_ISTORE_2:
			strcpy(out, "MNM_ISTORE_2");
			break;
		case MNM_ISTORE_3:
			strcpy(out, "MNM_ISTORE_3");
			break;
		case MNM_LSTORE_0:
			strcpy(out, "MNM_LSTORE_0");
			break;
		case MNM_LSTORE_1:
			strcpy(out, "MNM_LSTORE_1");
			break;
		case MNM_LSTORE_2:
			strcpy(out, "MNM_LSTORE_2");
			break;
		case MNM_LSTORE_3:
			strcpy(out, "MNM_LSTORE_3");
			break;
		case MNM_FSTORE_0:
			strcpy(out, "MNM_FSTORE_0");
			break;
		case MNM_FSTORE_1:
			strcpy(out, "MNM_FSTORE_1");
			break;
		case MNM_FSTORE_2:
			strcpy(out, "MNM_FSTORE_2");
			break;
		case MNM_FSTORE_3:
			strcpy(out, "MNM_FSTORE_3");
			break;
		case MNM_DSTORE_0:
			strcpy(out, "MNM_DSTORE_0");
			break;
		case MNM_DSTORE_1:
			strcpy(out, "MNM_DSTORE_");
			break;
		case MNM_DSTORE_2:
			strcpy(out, "MNM_DSTORE_2");
			break;
		case MNM_DSTORE_3:
			strcpy(out, "MNM_DSTORE_3");
			break;
		case MNM_ASTORE_0:
			strcpy(out, "MNM_ASTORE_0");
			break;
		case MNM_ASTORE_1:
			strcpy(out, "MNM_ASTORE_1");
			break;
		case MNM_ASTORE_2:
			strcpy(out, "MNM_ASTORE_2");
			break;
		case MNM_ASTORE_3:
			strcpy(out, "MNM_ASTORE_3");
			break;
		case MNM_IASTORE:
			strcpy(out, "MNM_IASTORE");
			break;
		case MNM_LASTORE:
			strcpy(out, "MNM_LASTORE");
			break;
		case MNM_FASTORE:
			strcpy(out, "MNM_FASTORE");
			break;
		case MNM_DASTORE:
			strcpy(out, "MNM_DASTORE");
			break;
		case MNM_AASTORE:
			strcpy(out, "MNM_AASTORE");
			break;
		case MNM_BASTORE:
			strcpy(out, "MNM_BASTORE");
			break;
		case MNM_CASTORE:
			strcpy(out, "MNM_CASTORE");
			break;
		case MNM_SASTORE:
			strcpy(out, "MNM_SASTORE");
			break;
		case MNM_POP:
			strcpy(out, "MNM_POP");
			break;
		case MNM_POP2:
			strcpy(out, "MNM_POP2");
			break;
		case MNM_DUP:
			strcpy(out, "MNM_DUP");
			break;
		case MNM_DUP_X1:
			strcpy(out, "MNM_DUP_X1");
			break;
		case MNM_DUP_X2:
			strcpy(out, "MNM_DUP_X2");
			break;
		case MNM_DUP2:
			strcpy(out, "MNM_DUP2");
			break;
		case MNM_DUP2_X1:
			strcpy(out, "MNM_DUP2_X1");
			break;
		case MNM_DUP2_X2:
			strcpy(out, "MNM_DUP2_X2");
			break;
		case MNM_SWAP:
			strcpy(out, "MNM_SWAP");
			break;
		case MNM_IADD:
			strcpy(out, "MNM_IADD");
			break;
		case MNM_LADD:
			strcpy(out, "MNM_LADD");
			break;
		case MNM_FADD:
			strcpy(out, "MNM_FADD");
			break;
		case MNM_DADD:
			strcpy(out, "MNM_DADD");
			break;
		case MNM_ISUB:
			strcpy(out, "MNM_ISUB");
			break;
		case MNM_LSUB:
			strcpy(out, "MNM_LSUB");
			break;
		case MNM_FSUB:
			strcpy(out, "MNM_FSUB");
			break;
		case MNM_DSUB:
			strcpy(out, "MNM_DSUB");
			break;
		case MNM_IMUL:
			strcpy(out, "MNM_IMUL");
			break;
		case MNM_LMUL:
			strcpy(out, "MNM_LMUL");
			break;
		case MNM_FMUL:
			strcpy(out, "MNM_FMUL");
			break;
		case MNM_DMUL:
			strcpy(out, "MNM_DMUL");
			break;
		case MNM_IDIV:
			strcpy(out, "MNM_IDIV");
			break;
		case MNM_LDIV:
			strcpy(out, "MNM_LDIV");
			break;
		case MNM_FDIV:
			strcpy(out, "MNM_FDIV");
			break;
		case MNM_DDIV:
			strcpy(out, "MNM_DDIV");
			break;
		case MNM_IREM:
			strcpy(out, "MNM_IREM");
			break;
		case MNM_LREM:
			strcpy(out, "MNM_LREM");
			break;
		case MNM_FREM:
			strcpy(out, "MNM_FREM");
			break;
		case MNM_DREM:
			strcpy(out, "MNM_DREM");
			break;
		case MNM_INEG:
			strcpy(out, "MNM_INEG");
			break;
		case MNM_LNEG:
			strcpy(out, "MNM_LNEG");
			break;
		case MNM_FNEG:
			strcpy(out, "MNM_FNEG");
			break;
		case MNM_DNEG:
			strcpy(out, "MNM_DNEG");
			break;
		case MNM_ISHL:
			strcpy(out, "MNM_ISHL");
			break;
		case MNM_LSHL:
			strcpy(out, "MNM_LSHL");
			break;
		case MNM_ISHR:
			strcpy(out, "MNM_ISHR");
			break;
		case MNM_LSHR:
			strcpy(out, "MNM_LSHR");
			break;
		case MNM_IUSHR:
			strcpy(out, "MNM_IUSHR");
			break;
		case MNM_LUSHR:
			strcpy(out, "MNM_LUSHR");
			break;
		case MNM_IAND:
			strcpy(out, "MNM_IAND");
			break;
		case MNM_LAND:
			strcpy(out, "MNM_LAND");
			break;
		case MNM_IOR:
			strcpy(out, "MNM_IOR");
			break;
		case MNM_LOR:
			strcpy(out, "MNM_LOR");
			break;
		case MNM_IXOR:
			strcpy(out, "MNM_IXOR");
			break;
		case MNM_LXOR:
			strcpy(out, "MNM_LXOR");
			break;
		case MNM_IINC:
			strcpy(out, "MNM_IINC");
			break;
		case MNM_I2L:
			strcpy(out, "MNM_I2L");
			break;
		case MNM_I2F:
			strcpy(out, "MNM_I2F");
			break;
		case MNM_I2D:
			strcpy(out, "MNM_I2D");
			break;
		case MNM_L2I:
			strcpy(out, "MNM_L2I");
			break;
		case MNM_L2F:
			strcpy(out, "MNM_L2F");
			break;
		case MNM_L2D:
			strcpy(out, "MNM_L2D");
			break;
		case MNM_F2I:
			strcpy(out, "MNM_F2I");
			break;
		case MNM_F2L:
			strcpy(out, "MNM_F2L");
			break;
		case MNM_F2D:
			strcpy(out, "MNM_F2D");
			break;
		case MNM_D2I:
			strcpy(out, "MNM_D2I");
			break;
		case MNM_D2L:
			strcpy(out, "MNM_D2L");
			break;
		case MNM_D2F:
			strcpy(out, "MNM_D2F");
			break;
		case MNM_I2B:
			strcpy(out, "MNM_I2B");
			break;
		case MNM_I2C:
			strcpy(out, "MNM_I2C");
			break;
		case MNM_I2S:
			strcpy(out, "MNM_I2S");
			break;
		case MNM_LCMP:
			strcpy(out, "MNM_LCMP");
			break;
		case MNM_FCMPL:
			strcpy(out, "MNM_FCMPL");
			break;
		case MNM_FCMPG:
			strcpy(out, "MNM_FCMPG");
			break;
		case MNM_DCMPL:
			strcpy(out, "MNM_DCMPL");
			break;
		case MNM_DCMPG:
			strcpy(out, "MNM_DCMPG");
			break;
		case MNM_IFEQ:
			strcpy(out, "MNM_IFEQ");
			break;
		case MNM_IFNE:
			strcpy(out, "MNM_IFNE");
			break;
		case MNM_IFLT:
			strcpy(out, "MNM_IFLT");
			break;
		case MNM_IFGE:
			strcpy(out, "MNM_IFGE");
			break;
		case MNM_IFGT:
			strcpy(out, "MNM_IFGT");
			break;
		case MNM_IFLE:
			strcpy(out, "MNM_IFLE");
			break;
		case MNM_IF_ICMPEQ:
			strcpy(out, "MNM_IF_ICMPEQ");
			break;
		case MNM_IF_ICMPNE:
			strcpy(out, "MNM_IF_ICMPNE");
			break;
		case MNM_IF_ICMPLT:
			strcpy(out, "MNM_IF_ICMPLT");
			break;
		case MNM_IF_ICMPGE:
			strcpy(out, "MNM_IF_ICMPGE");
			break;
		case MNM_IF_ICMPGT:
			strcpy(out, "MNM_IF_ICMPGT");
			break;
		case MNM_IF_ICMPLE:
			strcpy(out, "MNM_IF_ICMPLE");
			break;
		case MNM_IF_ACMPEQ:
			strcpy(out, "MNM_IF_ACMPEQ");
			break;
		case MNM_IF_ACMPNE:
			strcpy(out, "MNM_IF_ACMPNE");
			break;
		case MNM_GOTO:
			strcpy(out, "MNM_GOTO");
			break;
		case MNM_JSR:
			strcpy(out, "MNM_JSR");
			break;
		case MNM_RET:
			strcpy(out, "MNM_RET");
			break;
		case MNM_TABLESWITCH:
			strcpy(out, "MNM_TABLESWITCH");
			break;
		case MNM_LOOKUPSWITCH:
			strcpy(out, "MNM_LOOKUPSWITCH");
			break;
		case MNM_IRETURN:
			strcpy(out, "MNM_IRETURN");
			break;
		case MNM_LRETURN:
			strcpy(out, "MNM_LRETURN");
			break;
		case MNM_FRETURN:
			strcpy(out, "MNM_FRETURN");
			break;
		case MNM_DRETURN:
			strcpy(out, "MNM_DRETURN");
			break;
		case MNM_ARETURN:
			strcpy(out, "MNM_ARETURN");
			break;
		case MNM_RETURN:
			strcpy(out, "MNM_RETURN");
			break;
		case MNM_GETSTATIC:
			strcpy(out, "MNM_GETSTATIC");
			break;
		case MNM_PUTSTATIC:
			strcpy(out, "MNM_PUTSTATIC");
			break;
		case MNM_GETFIELD:
			strcpy(out, "MNM_GETFIELD");
			break;
		case MNM_PUTFIELD:
			strcpy(out, "MNM_PUTFIELD");
			break;
		case MNM_INVOKEVIRTUAL:
			strcpy(out, "MNM_INVOKEVIRTUAL");
			break;
		case MNM_INVOKESPECIAL:
			strcpy(out, "MNM_INVOKESPECIAL");
			break;
		case MNM_INVOKESTATIC:
			strcpy(out, "MNM_INVOKESTATIC");
			break;
		case MNM_INVOKEINTERFACE:
			strcpy(out, "MNM_INVOKEINTERFACE");
			break;
		case MNM_XXXUNUSEDXXX:
			strcpy(out, "MNM_XXXUNUSEDXXX");
			break;
		case MNM_NEW:
			strcpy(out, "MNM_NEW");
			break;
		case MNM_NEWARRAY:
			strcpy(out, "MNM_NEWARRAY");
			break;
		case MNM_ANEWARRAY:
			strcpy(out, "MNM_ANEWARRAY");
			break;
		case MNM_ARRAYLENGTH:
			strcpy(out, "MNM_ARRAYLENGTH");
			break;
		case MNM_ATHROW:
			strcpy(out, "MNM_ATHROW");
			break;
		case MNM_CHECKCAST:
			strcpy(out, "MNM_CHECKCAST");
			break;
		case MNM_INSTANCEOF:
			strcpy(out, "MNM_INSTANCEOF");
			break;
		case MNM_MONITORENTER:
			strcpy(out, "MNM_MONITORENTER");
			break;
		case MNM_MONITOREXIT:
			strcpy(out, "MNM_MONITOREXIT");
			break;
		case MNM_WIDE:
			strcpy(out, "MNM_WIDE");
			break;
		case MNM_MULTIANEWARRAY:
			strcpy(out, "MNM_MULTIANEWARRAY");
			break;
		case MNM_IFNULL:
			strcpy(out, "MNM_IFNULL");
			break;
		case MNM_IFNONNULL:
			strcpy(out, "MNM_IFNONNULL");
			break;
		case MNM_GOTO_W:
			strcpy(out, "MNM_GOTO_W");
			break;
		case MNM_JSR_W:
			strcpy(out, "MNM_JSR_W");
			break;
		// pseudo

		case MNM_FIRST_PSEUDO:
			strcpy(out, "MNM_FIRST_PSEUDO");
			break;

//		case MNM_PSEUDO_START:
//			strcpy(out, "MNM_PSEUDO_START");

		case MNM_PSEUDO_END:
			strcpy(out, "MNM_PSEUDO_END");
			break;

		case MNM_PSEUDO_TARGET:
			strcpy(out, "MNM_PSEUDO_TARGET");
			break;

		case MNM_PSEUDO_TRY:
			strcpy(out, "MNM_PSEUDO_TRY");
			break;

		case MNM_PSEUDO_END_TRY:
			strcpy(out, "MNM_PSEUDO_END_TRY");
			break;

		case MNM_PSEUDO_FINALLY:
			strcpy(out, "MNM_PSEUDO_FINALLY");
			break;

		case MNM_PSEUDO_END_FINALLY:
			strcpy(out, "MNM_PSEUDO_END_FINALLY");
			break;

		case MNM_PSEUDO_CATCH:
			strcpy(out, "MNM_PSEUDO_CATCH");
			break;

		case MNM_PSEUDO_END_CATCH:
			strcpy(out, "MNM_PSEUDO_END_CATCH");
			break;

		case MNM_PSEUDO_FAULT:
			strcpy(out, "MNM_PSEUDO_FAULT");
			break;

		case MNM_PSEUDO_END_FAULT:
			strcpy(out, "MNM_PSEUDO_END_FAULT");
			break;

		case MNM_PSEUDO_FILTER:
			strcpy(out, "MNM_PSEUDO_FILTER");
			break;

		case MNM_PSEUDO_END_FILTER:
			strcpy(out, "MNM_PSEUDO_END_FILTER");
			break;

		case MNM_PSEUDO_ILLEGAL:
			strcpy(out, "MNM_PSEUDO_ILLEGAL");
			break;

		case MNM_LAST_PSEUDO:
			strcpy(out, "MNM_LAST_PSEUDO");
			break;

		case MNM_FIRST_MIE_SPECIAL:
			strcpy(out, "MNM_FIRST_MIE_SPECIAL");
			break;

//		case MNM_MIE_LOAD_INST_OFFSET:
//			strcpy(out, "MNM_MIE_LOAD_INST_OFFSET");

		case MNM_LAST_MIE_SPECIAL:
			strcpy(out, "MNM_LAST_MIE_SPECIAL");
			break;

		default:
			strcpy(out, "<UNKNOWN>");
			break;
	}
	return MRTE_RESULT_OK;
}

TResult CMIEInstruction::SetMnemonic(const EMnemonic newMnemonic)
{
    //TEST
    if (SEM_PSEUDO == m_semTag)
    {
        return MRTE_ERROR_NOT_SUPPORTED;
    }
    if (newMnemonic < 0 || newMnemonic >= MNM_LAST_PSEUDO)
    {
        return MRTE_ERROR_ILLEGAL_ARGUMENT;
    }
    TResult res = InternalSetMnemonic(newMnemonic);
    return res;
}

TResult CMIEInstruction::InternalSetMnemonic(const EMnemonic newMnemonic)
{
    CMIEInstructionDescr &descr = CMIEInstructionFactory::GetDescriptor(newMnemonic);
/*
    if (NULL == pDescr)
    {
        return MRTE_ERROR_ILLEGAL_ARGUMENT;
    }
*/
    CopyOnModification();
    m_mnem = newMnemonic;
    m_szName = descr.GetName();
    m_semTag = descr.GetSemantics();
    if (descr.GetOpCount() != m_uiOperandsCount)
    {
        // reallocate space for operands (old operands are discarded)
        unsigned int i;
        for (i = 0; i < m_uiOperandsCount; ++i)
        {
            delete m_vpOperands[i];
        }
        delete [] m_vpOperands;
        AllocateOperands(descr.GetOpCount());        
    }
    m_uiOperandsCount = descr.GetOpCount();
    return MRTE_RESULT_OK;
}
//
// Returns the operand(s) of the instruction
//
// Parameters:
//      - pOperand1 [out] - a caller allocated SOperand structure
//      - pOperand2 [out] - a caller allocated SOperand structure
//      - pOperand3 [out] - a caller allocated SOperand structure
//      - pOperand4 [out] - a caller allocated SOperand structure
//
// Returns:
//      MRTE_RESULT_OK : success
//      MRTE_ERROR_BUFFER_TOO_SHORT : the STargetsArray member in the 
//                                    caller's SOperand is not big enough.
//      MRTE_ERROR_ILLEGAL_ARGUMENT : at least one of the instruction operands 
//                                    cannot be filled. The reason can be: its 
//                                    pOperand buffer is NULL, or its internal
//                                    STargetArray is not allocated (for switch
//                                    instruction).
//      MRTE_RESULT_NO_OPERAND : instruction has no operand or not as many
//                               operands as the number of arguments
//
TResult CMIEInstruction::GetOperand(SOperand *pOperand1, 
                                    SOperand *pOperand2,
                                    SOperand *pOperand3,
                                    SOperand *pOperand4) const
{
    //TEST
    unsigned int uiOpCount = GetOperandsCount();
    if (0 == uiOpCount)
    {
        return MRTE_RESULT_NO_OPERAND;
    }
    if (SEM_PSEUDO == m_semTag)
    {
        return MRTE_ERROR_NOT_SUPPORTED;
    }
    // copy operands into the appropriate buffers
    TResult res;
    res = CopyOperand(pOperand1, m_vpOperands[0], false);
    CHECK_TRESULT(res);
    if (uiOpCount >= 2)
    {
        res = CopyOperand(pOperand2, m_vpOperands[1], false);
        CHECK_TRESULT(res);
    }
    if (uiOpCount >= 3)
    {
        res = CopyOperand(pOperand3, m_vpOperands[2], false);
        CHECK_TRESULT(res);
    }
    if (uiOpCount >= 4)
    {
        res = CopyOperand(pOperand4, m_vpOperands[3], false);
        CHECK_TRESULT(res);
    }
    return MRTE_RESULT_OK;
}

//
// Returns a pointers (not a copy) to the operand at the specified index.
//
// Parameters:
//      uiIndex :   operand index (0-based)
//
// Returns:
//      SOperand* : a pointer to the operand
//      NULL      : when no such operand exists
//
SOperand* CMIEInstruction::InternalGetOperand(unsigned int uiIndex) const
{
    if (uiIndex >= GetOperandsCount())
    {
        return NULL;
    }
    return m_vpOperands[uiIndex];
}

//
// Sets or replaces the operands of the instruction with new ones. The 
// caller has to make sure the new operand types match the instruction mnemonics
// or change the mnemonics.
//
// Parameters:
//      pOperand1 [in] - the caller allocated SOperand 
//                       structure for the 1st operand.
//      pOperand2 [in] - the caller allocated SOperand 
//                       structure for the 2nd operand.
//      pOperand3 [in] - the caller allocated SOperand 
//                       structure for the 3rd operand.
//      pOperand4 [in] - the caller allocated SOperand 
//                       structure for the 4th operand.
//
// NOTE: for branch instruction, pOperand1 must contain a pointer to the IInstruction
//       object which is the branch target. If the target is not a pseudo 
//       .target, the pseudo .target is automatically
//       added just before the targeted instruction.
//
// Returns:
//      MRTE_RESULT_OK               : success
//      MRTE_ERROR_ILLEGAL_ARGUMENT  : Operand type is invalid
//      MRTE_ERROR_ARGUMENT_MISMATCH : Operand type and operand value mismatch.
//                                     e.g. operand type is OT_TARGET or 
//                                     OT_TARGETS_ARRAY and the operand value 
//                                     contains invalid IInstruction pointer(s), 
//                                     operand type is OT_VAR_ID and the value
//                                     contains an invalid variable id
//
TResult CMIEInstruction::SetOperand(SOperand *pOperand1, 
                                    SOperand *pOperand2, 
                                    SOperand *pOperand3,
                                    SOperand *pOperand4)
{
    //TEST
    if (SEM_PSEUDO == m_semTag)
    {
        return MRTE_ERROR_NOT_SUPPORTED;
    }
    if (NULL == pOperand1)
    {
        return MRTE_ERROR_ILLEGAL_ARGUMENT;
    }
    if (IsBranch())
    {
        // verify that the first operand is a valid branch target
        if (pOperand1->type != OT_TARGET)
        {
            return MRTE_ERROR_ILLEGAL_ARGUMENT;
        }
        IInstruction *pTarget = SetBranchTarget(pOperand1->val.pTarget);
        if (NULL == pTarget)
        {
            return MRTE_ERROR_ILLEGAL_ARGUMENT;
        }
        else
        {
            // branch target succesfully updated
            return MRTE_RESULT_OK;
        }
    } // if (IsBranch)
    CopyOnModification();
    InternalSetOperand(pOperand1, pOperand2, pOperand3, pOperand4);
    return MRTE_RESULT_OK;
}

TResult CMIEInstruction::AddBefore(IInstruction   **ppInstruction,
                                   const EMnemonic newInstMnemonic,
                                   const SOperand *pNewInstOperand1,
                                   const SOperand *pNewInstOperand2,
                                   const SOperand *pNewInstOperand3)
{
    //TEST
    if (NULL == m_pInstrumentor)
    {
        return MRTE_ERROR_FAIL;
    }
    if (IS_PSEUDO_MNEM(newInstMnemonic))
    {
        // pseudo-instructions cannot be added with this API
        return MRTE_ERROR_ILLEGAL_OPERATION;

    }
    if (MNM_PSEUDO_START == m_mnem)
    {
        // instructions cannot be added before .START
        return MRTE_ERROR_ILLEGAL_OPERATION;
    }
    CMIEInstruction *pNewInst;
    TResult res = m_pInstrumentor->AddInstructionBefore(&pNewInst,
        m_pfgNode, newInstMnemonic, pNewInstOperand1, pNewInstOperand2,
        pNewInstOperand3);
    *ppInstruction = pNewInst;
    return res;
}

TResult CMIEInstruction::AddAfter(IInstruction   **ppInstruction,
                                  const EMnemonic newInstMnemonic, 
                                  const SOperand *pNewInstOperand1,
                                  const SOperand *pNewInstOperand2,
                                  const SOperand *pNewInstOperand3)
{
    //TEST
    if (NULL == m_pInstrumentor)
    {
        return MRTE_ERROR_FAIL;
    }
    if (IS_PSEUDO_MNEM(newInstMnemonic))
    {
        // pseudo-instructions cannot be added with this API
        return MRTE_ERROR_ILLEGAL_OPERATION;

    }
    if (MNM_PSEUDO_END == m_mnem)
    {
        // instructions cannot be added after .END
        return MRTE_ERROR_ILLEGAL_OPERATION;
    }
    CMIEInstruction *pNewInst;
    TResult res = m_pInstrumentor->AddInstructionAfter(&pNewInst,
        m_pfgNode, newInstMnemonic, pNewInstOperand1, pNewInstOperand2,
        pNewInstOperand3);
    *ppInstruction = pNewInst;
    return res;
}

bool CMIEInstruction::IsOriginalInst() const
{
    return (IS_ORIGINAL == m_status);
}

TResult CMIEInstruction::GetOriginalOffset(unsigned int* puiOriginalOffset) const
{
    //TEST
    if (m_status == IS_NEW)
    {
        return MRTE_RESULT_ADDED_INSTRUCTION;
    }
    *puiOriginalOffset = m_uiOriginalOffset;
    return MRTE_RESULT_OK;
}

//
// Sets the branch target of this branch instruction.
//
// Parameters:
//      - pTargetInst [in] : the target instruction.
//                           If pTargetInst is not a pseudo .target, a pseudo .target
//                           will be automatically added just before pTargetInst.
//
// Returns:
//      - IInstruction* - the pseudo .target which is the branch target
//      - NULL - failure
//
IInstruction* CMIEInstruction::SetBranchTarget(IInstruction *pTargetInst)
{
    //TEST
    if (!IsBranch())
    {
        // the instruction object is not a branch
        return NULL;
    }
    if (NULL == pTargetInst)
    {
        return NULL;
    }
    if (GetOperandsCount() < 1)
    {
        return NULL;
    }

    CMIEInstruction *pMieTargetInst = (CMIEInstruction*)pTargetInst;
    CMIEInstruction *pPseudoTarget;
    if (pMieTargetInst->GetMnemonic() == MNM_PSEUDO_TARGET)
    {
        pPseudoTarget = pMieTargetInst;
    }
    else
    {
        // if we reached here, pTargetInst must mot be a pseudo instruction
        if (IS_PSEUDO_MNEM(pMieTargetInst->GetMnemonic()))
        {
            return NULL;
        }

        // if the instruction that precedes pTargetInst is a .target, use it
        CMIEInstruction *pInstBeforeTarget = 
            m_pInstrumentor->GetFlowGraph().GetPrecedingInstruction(pMieTargetInst);
        if (pInstBeforeTarget->GetMnemonic() == MNM_PSEUDO_TARGET)
        {
            pPseudoTarget = pInstBeforeTarget;
        }
        else
        {
            // create a new pseudo .target instruction
            TResult res = m_pInstrumentor->AddInstructionBefore(&pPseudoTarget,
                pMieTargetInst->GetFlowGraphNode(), MNM_PSEUDO_TARGET, NULL, NULL, NULL);
            if (res != MRTE_RESULT_OK)
            {
                return NULL;
            }
            SOperand opRealTargetInst;
            opRealTargetInst.type = OT_TARGET;
            opRealTargetInst.val.pTarget = pMieTargetInst;
            pPseudoTarget->InternalSetOperand(&opRealTargetInst);
        }
        
    }
    // update operand
    CopyOnModification();
    m_vpOperands[0]->type = OT_TARGET;
    m_vpOperands[0]->val.pTarget = pPseudoTarget;

    return pPseudoTarget;
}

IInstruction* CMIEInstruction::GetBranchTarget() const
{
    //TEST
    if (!IsBranch())
    {
        // the instruction object is not a branch
        return NULL;
    }
    if (GetOperandsCount() < 1)
    {
        return NULL;
    }
    if (m_vpOperands[0]->type != OT_TARGET)
    {
        return NULL;
    }
    IInstruction *pTargetInst = m_vpOperands[0]->val.pTarget;
    return pTargetInst;
}

//
// Copies the specified operands to the internal operands array of the CMIEInstruction
// object. The function does not perform any validity checks and assume that the 
// operands array is already initialized with the correct number of operands.
//
void CMIEInstruction::InternalSetOperand(const SOperand *pOperand1, 
                                         const SOperand *pOperand2, 
                                         const SOperand *pOperand3,
                                         const SOperand *pOperand4)
{
    // here we assume that the m_vpOperands array is correctly initialized.
    // this is done during CMIEInstruction construction (see CMIEInstructionFactory)
    if (m_uiOperandsCount >= 1)
    {
        CopyOperand(m_vpOperands[0], pOperand1, true);
    }
    if (m_uiOperandsCount >= 2)
    {
        CopyOperand(m_vpOperands[1], pOperand2, true);
    }
    if (m_uiOperandsCount >= 3)
    {
        CopyOperand(m_vpOperands[2], pOperand3, true);
    }
    if (m_uiOperandsCount >= 4)
    {
        CopyOperand(m_vpOperands[3], pOperand4, true);
    }
}

//
// Copies the specified operand to the specified location in the internal operands array
// of the CMIEInstruction object. The function does not perform any validity checks and 
// assume that the operands array is already initialized with the correct number of 
// operands.
//
void CMIEInstruction::InternalSetOperand(unsigned int uiOpInd,
                                         const SOperand *pOp)
{
    // here we assume that the m_vpOperands array is correctly initialized.
    // this is done during CMIEInstruction construction (see CMIEInstructionFactory)
    CopyOperand(m_vpOperands[uiOpInd], pOp, true);
}

void CMIEInstruction::GetOperatorDebugString(bool bToResolveNames,
                                             char *szOpValues)
{
    char szOpVal[1000];
    szOpValues[0] = 0;
    unsigned int i;
    for (i = 0; i < GetOperandsCount(); ++i)
    {
        OperandToString(szOpVal, m_vpOperands[i], bToResolveNames);
        strcat(szOpValues, szOpVal);
        strcat(szOpValues, " ");
    }
}

void CMIEInstruction::OperandToString(char *szOp, SOperand *pOp, bool bToResolveNames)
{
    if (NULL == pOp)
    {
        szOp[0] = 0;
        return;
    }

    unsigned int uiOffset;
    char szTemp[100];
    CMIEInstruction *pInst;
    unsigned int i;
	
    switch (pOp->type) {
    case OT_INT8:
		sprintf(szOp, "%d", pOp->val.s8Op);
        break;
    case OT_UINT8:
		sprintf(szOp, "%u", pOp->val.ui8Op);
        break;
    case OT_INT16:
		sprintf(szOp, "%d", pOp->val.s16Op);
        break;
    case OT_UINT16:
		sprintf(szOp, "%u", pOp->val.ui16Op);
        break;
    case OT_INT32:
		sprintf(szOp, "%d", pOp->val.s32Op);
        break;
    case OT_UINT32:
		sprintf(szOp, "%u", pOp->val.ui32Op);
        break;
    case OT_INT64:
		sprintf(szOp, "%I64d", pOp->val.s64Op);
        break;
    case OT_UINT64:
		sprintf(szOp, "%I64u", pOp->val.ui64Op);
        break;
    case OT_FLOAT32:
		sprintf(szOp, "%f", pOp->val.f32Op);
        break;
    case OT_FLOAT64:
		sprintf(szOp, "%f", pOp->val.d64Op);
        break;
    case OT_TARGET:
        pInst = (CMIEInstruction*)pOp->val.pTarget;
        if (NULL == pInst)
        {
            strcpy(szOp, "N/A");
        }
        else
        {
            if (pInst->IsPseudo())
            {
                sprintf(szOp, "(L%d)", pInst->GetLabel());
            }
            else
            {
                uiOffset = pInst->InternalGetOriginalOffset();
				sprintf(szOp, "%u", uiOffset);
            }
        }
        break;
    case OT_TARGETS_ARRAY:
        // process default target
        pInst = (CMIEInstruction*)pOp->val.targetsArray.pDefaultTarget;
        if (pInst != NULL)
        {
            uiOffset = pInst->InternalGetOriginalOffset();
			sprintf(szOp, "%u", uiOffset);
        }
        else
        {
            strcpy(szOp, "N/A");
        }
        // process other targets
        for (i = 0; i < pOp->val.targetsArray.uiUsedEntries; ++i)
        {
            pInst = (CMIEInstruction*)pOp->val.targetsArray.pTargetsArray[i].pTarget;
            if (pInst != NULL)
            {
                uiOffset = ((CMIEInstruction*)pInst)->InternalGetOriginalOffset();
			    sprintf(szTemp, "%u", uiOffset);
                strcat(szOp, " ");
                strcat(szOp, szTemp);
            }
            else
            {
                strcpy(szOp, "N/A");
            }
        }
        break;
    case OT_VAR_ID:
		sprintf(szOp, "%d", pOp->val.varID);
        break;
    case OT_JAVA_CP_INDEX:
    	sprintf(szOp, "0x%04x", pOp->val.cpIndex);
        if (bToResolveNames)
        {
            //TODO: implement CP item resolution
        }
        break;
    default:
        szOp[0] = 0;
    }
}

//
// Copies pSourceOp to pDestOp.
//
// Parameters:
//      pDestOp             [out] : a buffer allocated by the caller, that will contain the data
//                                  in pSourceOp
//      pSourceOp           [in]  : operand to copy
//      bAllocSwitchTargets [in]  : whether to allocate the targets array member when 
//                                  the operand is a OT_TARGETS_ARRAY and the targets
//                                  array is not big enough
//
// Returns:
//      MRTE_RESULT_OK : success
//      MRTE_ERROR_BUFFER_TOO_SHORT : the STargetsArray member in the 
//                                    pDestOp is not big enough.
//      MRTE_ERROR_ILLEGAL_ARGUMENT : a required member of pDestOp is not properly
//                                    allocated, or pDestOp is NULL.
//      MRTE_RESULT_NO_OPERAND      : pSourceOp is NULL and pDestOp is not NULL.
//
TResult CMIEInstruction::CopyOperand(SOperand *pDestOp, 
                                     const SOperand *pSourceOp,
                                     bool bAllocSwitchTargets) const
{
    if (NULL == pDestOp && pSourceOp != NULL)
    {
        return MRTE_ERROR_ILLEGAL_ARGUMENT;
    }
    if (NULL == pSourceOp && pDestOp != NULL)
    {
        return MRTE_RESULT_NO_OPERAND;
    }
    if (NULL == pSourceOp)
    {
        // both operands are NULL. this is ok. nothing to do.
        return MRTE_RESULT_OK;
    }
    // update operand type
    pDestOp->type = pSourceOp->type;
    // update operand data
    unsigned int i;
    switch (pSourceOp->type)
    {
    case OT_NO_OPERAND:
    case OT_INT8:      
    case OT_UINT8:     
    case OT_INT16:     
    case OT_UINT16:    
    case OT_INT32:     
    case OT_UINT32:    
    case OT_INT64:     
    case OT_UINT64:    
    case OT_FLOAT32:   
    case OT_FLOAT64:   
    case OT_TARGET:    
    case OT_VAR_ID:    
    case OT_DOT_NET_TOKEN:
    case OT_JAVA_CP_INDEX:
        pDestOp->val = pSourceOp->val;
        break;
    case OT_TARGETS_ARRAY:
        if (NULL == pSourceOp->val.targetsArray.pTargetsArray
            && 0 != pSourceOp->val.targetsArray.uiUsedEntries)
        {
            return MRTE_ERROR_ILLEGAL_ARGUMENT;
        }
        if (pDestOp->val.targetsArray.uiEntries < pSourceOp->val.targetsArray.uiUsedEntries)
        {
            if (bAllocSwitchTargets)
            {
                pDestOp->val.targetsArray.uiEntries = 
                    pSourceOp->val.targetsArray.uiUsedEntries;
                pDestOp->val.targetsArray.uiUsedEntries = 
                    pSourceOp->val.targetsArray.uiUsedEntries;
                pDestOp->val.targetsArray.pTargetsArray = new SCaseTarget[
                    pDestOp->val.targetsArray.uiUsedEntries];
            }
            else
            {
                pDestOp->val.targetsArray.uiUsedEntries = 
                    pSourceOp->val.targetsArray.uiUsedEntries;
                return MRTE_ERROR_BUFFER_TOO_SHORT;
            }
        }
        // copy array members
        pDestOp->val.targetsArray.uiUsedEntries = pSourceOp->val.targetsArray.uiUsedEntries;
        pDestOp->val.targetsArray.pDefaultTarget = pSourceOp->val.targetsArray.pDefaultTarget;
        for (i = 0 ; i < pSourceOp->val.targetsArray.uiUsedEntries; ++i)
        {
            pDestOp->val.targetsArray.pTargetsArray[i] = 
                pSourceOp->val.targetsArray.pTargetsArray[i];

        }
        break;
    default:
        // unknown operand type
        return MRTE_ERROR_ILLEGAL_ARGUMENT;
    } // switch (pSourceOp->type)
    return MRTE_RESULT_OK;
}

//
// Updates the instruction's operand with the branch target.
//
// Parameters:
//      - pTargetInst : target instruction. Must be a pseudo .target
//
// Returns:
//      - MRTE_RESULT_OK : success
//      - MRTE_ERROR_ILLEGAL_ARGUMENT: pTargetInst is NULL or not a pseudo .target
//      - MRTE_ERROR_FAIL : unknown failure
//
TResult CMIEInstruction::InternalSetBranchTarget(CMIEInstruction *pTargetInst)
{
    if (NULL == pTargetInst)
    {
        return MRTE_ERROR_ILLEGAL_ARGUMENT;
    }
    if (pTargetInst->GetMnemonic() != MNM_PSEUDO_TARGET)
    {
        return MRTE_ERROR_ILLEGAL_ARGUMENT;
    }
    // update the instruction's operand
    if (GetOperandsCount() < 1)
    {
        return MRTE_ERROR_FAIL;
    }
    m_vpOperands[0]->type = OT_TARGET;
    m_vpOperands[0]->val.pTarget = pTargetInst;
    return MRTE_RESULT_OK;
}

//
// Updates the instruction's operand with the branch target.
//
// Parameters:
//      - uiTargetIndex : the target to update
//      - pTargetInst : target instruction. Must be a pseudo .target
//
// Returns:
//      - MRTE_RESULT_OK : success
//      - MRTE_ERROR_ILLEGAL_ARGUMENT: pTargetInst is NULL or not a pseudo .target
//      - MRTE_ERROR_FAIL : unknown failure
//
TResult CMIEInstruction::InternalSetSwitchTarget(unsigned int uiTargetIndex,
                                                 CMIEInstruction *pTargetInst)
{
    if (NULL == pTargetInst)
    {
        return MRTE_ERROR_ILLEGAL_ARGUMENT;
    }
    if (pTargetInst->GetMnemonic() != MNM_PSEUDO_TARGET)
    {
        return MRTE_ERROR_ILLEGAL_ARGUMENT;
    }
    // update the instruction's operand
    if (GetOperandsCount() < 1)
    {
        return MRTE_ERROR_FAIL;
    }
    m_vpOperands[0]->type = OT_TARGETS_ARRAY;
    m_vpOperands[0]->val.targetsArray.pTargetsArray[uiTargetIndex].pTarget = pTargetInst;
    return MRTE_RESULT_OK;
}

TResult CMIEInstruction::InternalSetSwitchDefaultTarget(CMIEInstruction *pTargetInst)
{
    if (NULL == pTargetInst)
    {
        return MRTE_ERROR_ILLEGAL_ARGUMENT;
    }
    if (pTargetInst->GetMnemonic() != MNM_PSEUDO_TARGET)
    {
        return MRTE_ERROR_ILLEGAL_ARGUMENT;
    }
    // update the instruction's operand
    if (GetOperandsCount() < 1)
    {
        return MRTE_ERROR_FAIL;
    }
    m_vpOperands[0]->type = OT_TARGETS_ARRAY;
    m_vpOperands[0]->val.targetsArray.pDefaultTarget = pTargetInst;
    return MRTE_RESULT_OK;
}

//
// Allocates and initializes the internal operands array
//
void CMIEInstruction::AllocateOperands(unsigned int uiOpCount)
{
    m_uiOperandsCount = uiOpCount;
    if (0 == uiOpCount) 
    {
        // no operands
        return;
    }
    m_vpOperands = new SOperand*[uiOpCount];
    unsigned int i;
    for (i = 0; i < uiOpCount; ++i)
    {
        SOperand *pNewOP = new SOperand();
        pNewOP->type = OT_NO_OPERAND; // default type
        pNewOP->val.targetsArray.uiEntries = 0;
        pNewOP->val.targetsArray.uiUsedEntries = 0;
        pNewOP->val.targetsArray.pTargetsArray = NULL;
        m_vpOperands[i] = pNewOP;
    }
}

CMIEReadOnlyInstruction* CMIEInstruction::GetReadOnlyWrapper()
{
    if (NULL == m_pReadOnlyInst)
    {
        m_pReadOnlyInst = new CMIEReadOnlyInstruction(this);
    }
    return m_pReadOnlyInst;
}

bool CMIEInstruction::CopyOnModification()
{
    CFlowGraph &flowGraph = m_pInstrumentor->GetFlowGraph();
    bool res = flowGraph.CopyOnModification(this);
    if (!res)
    {
        return false;
    }
    // update the status of the current instruction
    if (IS_ORIGINAL == m_status)
    {
        m_status = IS_MODIFIED;
    }
    return true;
}

//
// Creates and returns an exact copy of the instruction object.
//
// Returns:
//      CMIEInstruction*    - an exact copy of the instruction object
//      NULL                - error (memory allocation problem)
//
CMIEInstruction* CMIEInstruction::Clone()
{
    CMIEInstruction *pClonedInst = 
        CMIEInstructionFactory::CreateInstruction(m_mnem, m_pInstrumentor);
    CHECK_RETURN_NULL(pClonedInst);
    pClonedInst->m_status = m_status;
    pClonedInst->m_semTag = m_semTag;
    pClonedInst->m_szName = m_szName;
    pClonedInst->m_uiOriginalOffset = m_uiOriginalOffset;
    pClonedInst->m_uiTargetOffset = m_uiTargetOffset;
    pClonedInst->m_uiTargetsCount = m_uiTargetsCount;
    // copy targets array (for switch instructions)
    unsigned int i;
    if (m_uiTargetsCount > 0)
    {
        pClonedInst->m_vuiTargetOffsets = new unsigned int[m_uiTargetsCount];
        for (i = 0; i < m_uiTargetsCount; ++i)
        {
            pClonedInst->m_vuiTargetOffsets[i] = m_vuiTargetOffsets[i];
        }
    }
    // copy operands
    for (i = 0; i < m_uiOperandsCount; ++i)
    {
        CopyOperand(pClonedInst->m_vpOperands[i], m_vpOperands[i], true);
    }
    return pClonedInst;
}

bool CMIEInstruction::IsOpCountValid(const SOperand *pOp1,
                                     const SOperand *pOp2,
                                     const SOperand *pOp3,
                                     const SOperand *pOp4)
{
    unsigned int uiOpCount = 0;
    if (NULL != pOp1)
    {
        uiOpCount++;
    }
    if (NULL != pOp2)
    {
        uiOpCount++;
    }
    if (NULL != pOp3)
    {
        uiOpCount++;
    }
    if (NULL != pOp4)
    {
        uiOpCount++;
    }
    return uiOpCount == m_uiOperandsCount;
}

// Calculate padding bytes for switch instructions
inline static unsigned CalcPad(unsigned int ip)	
{
	return ++ip % 4 ? 4 - ip % 4 : 0;
}

//
// Calculates and returns the instruction size in bytes.
//
// Parameters:
//      uiCurrentIP :   the current instruction pointer. This is required because
//                      it may affect the instruction size. For example, the Java
//                      tableswitch and lookupswitch instructions use padding
//                      to align their jump tables.
//
unsigned int CMIEInstruction::GetSize(unsigned int uiCurrentIP)
{
    if (IsPseudo())
    {
        return 0;
    }

    unsigned int uiSize;
    int align;
    SOperand *pOp1;
    SOperand *pOp2;
    short constVal;
    unsigned int i;

    switch (m_mnem)
    {
    case MNM_TABLESWITCH:
        // calculate the size of the tableswitch instruction
	    align = CalcPad(uiCurrentIP);
	    uiSize =  1 + align + 12 + (m_uiHigh - m_uiLow + 1) * 4;
    	break;
    case MNM_LOOKUPSWITCH:
        // calculate the size of the lookupswitch instruction
        align = CalcPad(uiCurrentIP);
        uiSize = 1 + align + 8 + m_uiTargetsCount * 8;
    	break;
    case MNM_ILOAD:
    case MNM_LLOAD:
    case MNM_ALOAD:
    case MNM_DLOAD:
    case MNM_FLOAD:
    case MNM_ISTORE:
    case MNM_LSTORE:
    case MNM_ASTORE:
    case MNM_DSTORE:
    case MNM_FSTORE:
    case MNM_RET:
        // the size depends on whether the regular version or the wide version is used
        pOp1 = InternalGetOperand(0);
        assert(OT_VAR_ID == pOp1->type);
	    if (pOp1->val.varID > 255)
        {
            // wide version
            uiSize = 4;
        }
        else
        {
            // regular version
            uiSize = 2;
        }
        break;
    case MNM_IINC:
        // the size depends on whether the regular version or the wide version is used
        pOp1 = InternalGetOperand(0);
        pOp2 = InternalGetOperand(1);
        assert(OT_VAR_ID == pOp1->type);
        assert(OT_INT8 == pOp2->type || OT_INT16 == pOp2->type);

       	#if defined(_SOLARIS) || defined(_SOLARISX86)
       		// These lines on Solaris, and the line below, should produce exactly the same result. However, these
       		// produce different results even when values are explicitly casted and extra parentheses are liberally applied. - jgw
       		//
	        if(OT_INT8 == pOp2->type) {
	        	constVal = (pOp2->val.s8Op);
	        } else {
	        	constVal = (pOp2->val.s16Op);
	        }
        #else
        	constVal = OT_INT8 == pOp2->type ? pOp2->val.s8Op : pOp2->val.s16Op;
        #endif


	    if (pOp1->val.varID > 255 || constVal > 127 || constVal < -128)
        {
            // wide version
            uiSize = 6;
        }
        else
        {
            // regular version
            uiSize = 3;
        }
        break;
    case MNM_GOTO_W:
    case MNM_JSR_W:
        uiSize = 5;
        break;
    case MNM_LDC:
        uiSize = 2;
        break;
    case MNM_MIE_LOAD_INST_OFFSET:
        // loadinstoffset is implemented as ldc2_w
        uiSize = 3;
        break;
    default:
        // generic calculation based on the instruction operands
        uiSize = 1;
        for (i = 0; i < m_uiOperandsCount; ++i)
        {
            pOp1 = InternalGetOperand(i);
            switch (pOp1->type)
            {
            case OT_NO_OPERAND:
                uiSize += 0;
    	        break;
            case OT_INT8:
            case OT_UINT8:
                uiSize += 1;
                break;
            case OT_INT16:
            case OT_UINT16:
            case OT_TARGET:
                // by default, target is a 16 bit value (wide goto and jsr are
                // handled separately)
            case OT_JAVA_CP_INDEX:
                uiSize += 2;
                break;
            case OT_INT32:
            case OT_UINT32:
            case OT_FLOAT32:
                uiSize += 4;
                break;
            case OT_INT64:
            case OT_UINT64:
            case OT_FLOAT64:
                uiSize += 8;
                break;
            } // of switch (operand type)
        } // for
    } // end switch
    return uiSize;
}

void CMIEInstruction::DeallocateOperand(SOperand *pOp)
{
    if (OT_TARGETS_ARRAY == pOp->type)
    {
        delete [] pOp->val.targetsArray.pTargetsArray;
    }
}

